/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include "blade-config.h"
#include <iostream>

#include <QLabel>
#include <QGraphicsView>
#include <QGraphicsScene>
#include <QRectF>
#include <QVBoxLayout>
#include <QScrollArea>
#include <QAction>

#include "Service.h"
#include "BladePluginWidget.h"


BladePluginWidget::BladePluginWidget(QString path, cube::Cube* _cube, BladeToolBar * _toolBar,  QWidget * parent):QWidget(parent),cube(_cube),toolBar(_toolBar)
{
    toolBar->setBladeWidget(NULL);
    blade_widget = NULL;
    QVBoxLayout* layout = new QVBoxLayout();
    this->setLayout( layout );
    
    _stacked_widget = new QStackedWidget();
    layout->addWidget(_stacked_widget);
    
    load_trace_widget = new LoadTraceWidget(path, cube);
    _stacked_widget->addWidget(load_trace_widget);
    connect( load_trace_widget, SIGNAL( traceLoaded()), this, SLOT(showTrace()));
}
    

    
void
BladePluginWidget::showTrace()
{
    blade_widget = new BladeWidget( load_trace_widget->getTrace(), cube);
    toolBar->setBladeWidget(blade_widget);
    toolBar->show();
    _stacked_widget->addWidget(blade_widget);
    _stacked_widget->setCurrentWidget(blade_widget);
}
    
void
BladePluginWidget::selectionChanged( cube::Metric* _m, bool _m_is_expanded,
                               cube::Cnode* _cp, bool _c_is_expanded,
                               cube::Sysres * _s, bool _s_is_expanded,
                               cube::Region * _r, bool _r_is_expanded)
{
    if (blade_widget != NULL)
    {
        blade_widget->selectionChanged(_m, _m_is_expanded, 
                                        _cp, _c_is_expanded,
                                        _s, _s_is_expanded,
                                        _r, _r_is_expanded );
    }
}
     
void
BladePluginWidget::tabActivated( cubepluginapi::TreeType type )
{
    if (blade_widget != NULL)
    {
        blade_widget->tabActivated( type );
    }
}
  
    
    
    
