/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include "blade-config.h"
#include "BladeToolBar.h"
#include "PluginServices.h"
#include <QToolButton>
#include <QMessageBox>
#include <QInputDialog>
#include <QMenu>
#include <assert.h>

#include "RoleFilter.h"
#include "RegionPropertiesFilter.h"

extern RoleFilter * role_filter;
extern RegionPropertiesFilter * rp_filter;

BladeToolBar::BladeToolBar( cubepluginapi::PluginServices* service ) : QToolBar( "Blade Toolbar" )
{
    parent          = 0;
    this->service   = service;

    
    info_action = this->addAction(QIcon(":/bladeicons/info.png"),"Trace Info");
    
    connect(info_action, SIGNAL(triggered()),
	    this, SLOT(info()));   
    
    this->addSeparator();
    
    back_action = this->addAction(QIcon(":/bladeicons/back.png"),"Back");        
    connect(back_action, SIGNAL(triggered()),
	    this, SLOT(goBack()));


    left_action = this->addAction(QIcon(":/bladeicons/left.png"),"Move left");
    connect(left_action, SIGNAL(triggered()),
	    this, SLOT(moveLeft()));

    unzoom_action = this->addAction(QIcon(":/bladeicons/unzoom.png"),"Unzoom");        
    connect(unzoom_action, SIGNAL(triggered()),
	    this, SLOT(unZoom()));

    
    right_action = this->addAction(QIcon(":/bladeicons/right.png"),"Move right");
    connect(right_action, SIGNAL(triggered()),
	    this, SLOT(moveRight()));      
    
    this->addSeparator();
    
    compact_action = this->addAction(QIcon(":/bladeicons/compact.png"),"Compact view");
    connect(compact_action, SIGNAL(triggered()),
	    this, SLOT(compactToggle()));  
    
    roles_filter_action = this->addAction(QIcon(":/bladeicons/roles_filter.png"),"Roles filter");
    QMenu * filter_menu = new QMenu();
    QAction * collective = filter_menu->addAction("Collective");
    collective->setCheckable(true);
    collective->setChecked(true);
    connect(collective, SIGNAL(toggled(bool)),
	    this, SLOT(selectCollective(bool)));
    QAction * point2point = filter_menu->addAction("Point-to-Point");
    point2point->setCheckable(true);
    point2point->setChecked(true);
    connect(point2point, SIGNAL(toggled(bool)),
	    this, SLOT(selectPoint2Point(bool)));
    QAction * others = filter_menu->addAction("Others");
    others->setCheckable(true);
    others->setChecked(true);
    connect(others, SIGNAL(toggled(bool)),
	    this, SLOT(selectOthers(bool)));        

    roles_filter_action->setMenu( filter_menu);

    this->addSeparator();   
    QComboBox * prop_selector = new QComboBox();
    prop_selector->addItem("Name");
    this->addWidget(prop_selector);
    
    
    filter_input = new QLineEdit();
    this->addWidget(filter_input);
    connect(filter_input, SIGNAL(textEdited(const QString&)),
	    this, SLOT(setRegionFilterName(const QString&)));     
    
        
      
}


void
BladeToolBar::setBladeWidget( BladeWidget* newparent )
{
    parent = newparent;
}


// slots connected to the actions in the topology tool bar
void
BladeToolBar::showSelf()
{
    if ( parent != NULL )
    {
        this->show();
    }
}

void
BladeToolBar::hideSelf()
{
    this->hide();
}


void
BladeToolBar::info()
{
    if ( parent != NULL )
    {
        parent->info();
    }
}
void
BladeToolBar::goBack()
{
    if ( parent != NULL )
    {
        parent->goBack();
    }
}
void
BladeToolBar::moveRight()
{
    if ( parent != NULL )
    {
        parent->moveRight();
    }
}
void
BladeToolBar::moveLeft()
{
    if ( parent != NULL )
    {
        parent->moveLeft();
    }
}

void
BladeToolBar::unZoom(  )
{
    if ( parent != NULL )
    {
        parent->unZoom( );
    }
}

void
BladeToolBar::compactToggle(  )
{
    if ( parent != NULL )
    {
        parent->compactToggle( );
    }
}


void
BladeToolBar::selectOthers( bool _status  )
{
    role_filter->set_others( _status );
}
void
BladeToolBar::selectCollective( bool _status  )
{
    role_filter->set_collective( _status );
}

void
BladeToolBar::selectPoint2Point( bool _status  )
{
    role_filter->set_point2point( _status );
}


void
BladeToolBar::setRegionFilterName( const QString & _text)
{
    rp_filter->set_name( _text.toStdString());
}
