/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include "blade-config.h"
#include <iostream>
#include <utility>
#include <limits>

#include <QLabel>
#include <QRectF>
#include <QHBoxLayout>
#include <QGraphicsItem>
#include <QInputDialog>
#include "MetricTraceWidget.h"
#include "Service.h"


MetricTraceWidget::MetricTraceWidget( QWidget * parent):QWidget(parent)
{
    margin = 10;
    metric_expression  = QString("delta(metric::PAPI_TOT_CYC())/delta(metric::time())");
    metric_line = new MetricLine();
    metric_line->set_expression( metric_expression );
    
    
    QHBoxLayout* layout = new QHBoxLayout();
    this->setLayout( layout );
    
    scene = new QGraphicsScene();
    view  = new MetricGraphicsView( scene, metric_line );

    layout->addWidget( view );
    
    setMaximumHeight(300);     
    setMinimumHeight(300);     
    updateScene();
    hide();

    
    connect(z_filter, SIGNAL(filterChanged()),
	    this, SLOT(updateScene())); 
    
    connect(view, SIGNAL(zoomChanged()),
	    this, SLOT(changeZoom()));

    connect( view, SIGNAL(zoomingSelection()),
		this, SLOT(zoomingTip()));
    connect( view, SIGNAL(cursorMoved(QPointF)),
		this, SLOT(updateCursor(QPointF)));      
    
    
    updateScene();
    
        
        
}
    
    
MetricTraceWidget::~MetricTraceWidget()
{
    delete metric_line;
}
    
void
MetricTraceWidget::updateCursor(QPointF _p)
{
    if (isHidden())
        return;
    emit cursorMoved(_p);
    
//     uint64_t _s = z_filter->get_time_in_window( _p.x());
//     std::pair<double, uint16_t> _s_in_seconds = trace->convert_to_seconds( z_filter->get_zoom_start() );
//     std::pair<double, uint16_t> _f_s_in_seconds = trace->convert_to_seconds( z_filter->get_zoom_end() );
//     std::pair<double, uint16_t> _p_in_seconds = trace->convert_to_seconds( _s );
//     blade_service->updateValueWidget(   NULL,
//                                         _s_in_seconds.first  ,
//                                         _f_s_in_seconds.first ,
//                                         _p_in_seconds.first,
//                                         0.,
//                                         0.,
//                                         0.,
//                                         0., 
//                                         0.
//                                     );
//     
//     
} 
    
    
QPointF
MetricTraceWidget::getCursor()
{
   return view->getCursorPosition();
} 
         
         
void
MetricTraceWidget::setCursor(QPointF& _p)
{
    if (isHidden())
        return;
    view->setCursorPosition( _p );
    
    cursorPosition = QPoint( _p.x() * view->width(),0) ;
    cursor->setLine(cursorPosition.x(),0, cursorPosition.x(), height() );
    
} 
             
    
    
void
MetricTraceWidget::changeZoom()
{
    QPointF _p = view->getZoomF();
    z_filter->zoom_in( _p );

    resizeEvent(0);
}
  
void
MetricTraceWidget::zoomingTip()
{
//     if (isHidden())
//         return;
//     QPointF _p = view->getZoomF();
//     uint64_t _s = z_filter->get_time_in_window( _p.x());
//     uint64_t _f = z_filter->get_time_in_window( _p.y());
//     std::pair<double, uint16_t> _s_in_seconds = trace->convert_to_seconds( _s );
//     std::pair<double, uint16_t> _f_s_in_seconds = trace->convert_to_seconds( _f - _s );
//     QString _tip = (QString(" From %1%2s").arg(_s_in_seconds.first).arg( QString::fromStdString(bladeservices::convertToUnit (_s_in_seconds.second)) ) +
//     QString("+%1%2s").arg(_f_s_in_seconds.first).arg( QString::fromStdString(bladeservices::convertToUnit (_f_s_in_seconds.second)), 0, 'f' )
//    );
//     view->showToolTip( _tip );
    
}
      
  
  
void 
MetricTraceWidget::resizeEvent(QResizeEvent *event)
{
    updateScene();
}



  
void 
MetricTraceWidget::addCounterLine()
{

     bool ok;
     QString _metric_expression = QInputDialog::getText(this, tr("Set derived metric"),
                                          tr("Expression:"), QLineEdit::Normal,
                                         metric_expression , &ok);
     if (ok )
     {
        metric_expression = _metric_expression;
        metric_line->set_expression( metric_expression );
        updateScene();
     }
    
}



void 
MetricTraceWidget::updateScene()
{
    QPen rect_pen;
    rect_pen.setColor( QColor( 0,0,0,50));
    
    
    scene->clear();
    QGraphicsRectItem * _rect = scene->addRect(QRectF(0  ,margin , view->width() , view->height()-2*margin), rect_pen);
    view->centerOn( QPointF(view->width()/2., view->height()/2.) );
    if (isHidden())
        return;
    zoom =  new  QGraphicsRectItem();
    cursor = new QGraphicsLineItem();   
    scene->addItem( zoom );
    scene->addItem( cursor );
    
    QPen pen;
    pen.setColor( QColor( 0,0,0,100));
    QBrush brush;
    brush.setColor( QColor( 0,0,0,100));
    brush.setStyle(Qt::SolidPattern);
    
    
    QPen cursor_pen;
    cursor_pen.setColor( QColor( 0,0,0,255));
    cursor_pen.setWidth(2);
    cursor->setPen( cursor_pen );
    
    
    zoom->setPen( pen );
    zoom->setBrush( brush );
    
    zoom->setRect( zoom_rect );
    
    
    QPen title_pen;
    title_pen.setColor( QColor( 0,0,0,50));
    
     
    metric_label = new QGraphicsSimpleTextItem(metric_expression);
    scene->addItem( metric_label );
    metric_label->setOpacity(0.2);
    metric_label->setPos(margin,view->height()-5*margin);
    
    

    std::vector<TraceEventStrippedSection> &_stripped_sections = metric_line->get_stripped_sections( events );
    
    
    double _min = std::numeric_limits<double>::max();
    double _max = std::numeric_limits<double>::min();

    for (std::vector<TraceEventStrippedSection>::iterator iter = _stripped_sections.begin(); iter != _stripped_sections.end(); ++iter)
    {
        TraceEventStrippedSection& _section = *iter;
         _min = std::min( _min, _section.start_value);
         _max = std::max( _max, _section.start_value);
         _min = std::min( _min, _section.end_value);
         _max = std::max( _max, _section.end_value);
    }
    
    double _n_min= (_max==_min)?_min-10.:_min;
    double _n_max= (_max==_min)?_max+10.:_max;
    _min =_n_min;
    _max =_n_max;
    
    
    double y_range = _max-_min;
    
    min_label = new QGraphicsSimpleTextItem(QString("%1").arg(_min));
    
    max_label = new QGraphicsSimpleTextItem(QString("%1").arg(_max));
  
    
    uint64_t _x1 = std::numeric_limits<double>::max();
    uint64_t _y1 = std::numeric_limits<double>::max();
    uint64_t _x2 = std::numeric_limits<double>::max();
    uint64_t _y2 = std::numeric_limits<double>::max();
    
    
    
    
    for (std::vector<TraceEventStrippedSection>::iterator iter = _stripped_sections.begin(); iter != _stripped_sections.end(); ++iter)
    {
        TraceEventStrippedSection& _section = *iter;
        double _s = _section.start_value;
        double _f = _section.end_value;
        
        double _b = (double)(_s-_min)/(double)y_range * (view->height()-2*margin);
        double _e = (double)(_f-_min)/(double)y_range * (view->height() -2*margin);
        
         double _normalized_start =  (_section.start_t > z_filter->get_zoom_start())?
                                    ((double)(_section.start_t - z_filter->get_zoom_start())
                                    /
                                    (double)( z_filter->get_zoom_size() )):0;
                                    
        double _normalized_end =    (_section.end_t  < z_filter->get_zoom_size()+ z_filter->get_zoom_start())?
                                    ((double)(_section.end_t -  z_filter->get_zoom_start())
                                    /
                                    (double)(z_filter->get_zoom_size() )):1.;
                                   
        std::pair<uint64_t, uint64_t> _point=std::make_pair( (uint64_t)(_normalized_start * (view->width())), (uint64_t)(_normalized_end * (view->width())));
        
        
        
        
        uint64_t _tmp_x1 = _point.first;
        uint64_t _tmp_y1 = view->height()-margin-_b;
        uint64_t _tmp_x2 = _point.second;
        uint64_t _tmp_y2 = view->height()-margin-_e;
        
        if (_tmp_x1 == _x2)
        {
            scene->addLine( _x2, _y2, _tmp_x1, _tmp_y1);
        }
        
        scene->addLine( _tmp_x1, _tmp_y1, _tmp_x2, _tmp_y2);
        
        _x1 = _tmp_x1;
        _y1 = _tmp_y1;
        _x2 = _tmp_x2;
        _y2 = _tmp_y2;
    }
    
    
    scene->addItem( min_label );
    min_label->setPos(margin,view->height()-3*margin);
    scene->addItem( max_label );
    max_label->setPos(margin,margin);
    
    
    
    
    view->updateScene();
}


void
MetricTraceWidget::set_trace_view( const std::vector<TraceEvent*> & _events)
{
    events = _events;
    if (isHidden() || metric_line == NULL)
        return;

    updateScene();
  
    
      
}




