/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include "blade-config.h"
#include "RoleFilter.h"



RoleFilter::RoleFilter()
{
    collective = true;
    p2p = true;
    others = true;
}


RoleFilter::~RoleFilter()
{
}


void
RoleFilter::set_others( bool _status  )
{
   others = _status;
   emit filterChanged();
}

void
RoleFilter::set_collective( bool _status  )
{
   collective = _status;
   emit filterChanged();
}

void
RoleFilter::set_point2point( bool _status  )
{
   p2p = _status;
   emit filterChanged();
}



FilterResult 
RoleFilter::apply( TraceEvent * _te )
{
    if (!is_active())
        return FILTER_OK;

    OTF2_RegionRole _role = _te->get_region_role();
    switch(_role)
    {
        case OTF2_REGION_ROLE_COLL_ONE2ALL :
        case OTF2_REGION_ROLE_COLL_ALL2ONE  :
        case OTF2_REGION_ROLE_COLL_ALL2ALL  :
        case OTF2_REGION_ROLE_COLL_OTHER  :
        case OTF2_REGION_ROLE_BARRIER   :
            if (!collective)
                return FILTER_CHILDREN;
            else
                return FILTER_OK;
            break;
        case OTF2_REGION_ROLE_POINT2POINT   :
            if (!p2p)
                return FILTER_CHILDREN;
            else
                return FILTER_OK;
            break;  
        default:
            if (!others)
                return FILTER_CHILDREN;
            else
                return FILTER_OK;
            break;  
    }

    return FILTER_FAIL;
    
}
