/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include "blade-config.h"
#include <iostream>
#include <utility>
#include <QIcon>
#include "TraceControl.h"
#include "Service.h"
#include "SubTraceWidget.h"


TraceControl::TraceControl( QWidget * parent):QToolBar(parent)
{
       hidden = true; 
       metrics_hidden = true;       
       hideToggle = addAction(QIcon(":/bladeicons/show.png"), "show threads"); 
       hideMetrics = addAction(QIcon(":/bladeicons/counterline.png"), "show counters"); 
       connect( hideToggle, SIGNAL( triggered( void )), this, SLOT(toggleHiding()));
       connect( hideMetrics, SIGNAL( triggered( void )), this, SLOT(toggleMetrics()));
}

    
    
TraceControl::~TraceControl()
{
}
    
void
TraceControl::toggleHiding()
{
    hidden = !hidden;
    if (hidden)
    {
        hideToggle->setIcon(QIcon(":/bladeicons/show.png"));
        hideToggle->setIconText("show threads");
    }
    else
    {
        hideToggle->setIcon(QIcon(":/bladeicons/hide.png"));
        hideToggle->setIconText("hide threads");
    }
    for (std::vector<QWidget *>::iterator iter = widgets.begin(); iter != widgets.end(); ++iter)
    {
        QWidget * _w = (*iter);
        SubTraceWidget * _as_sw = dynamic_cast<SubTraceWidget*>(_w);
        if (hidden)
        {
            (*iter)->hide();
        }
        else
        {
            (*iter)->show();
            if (_as_sw != NULL)
                _as_sw->updateScene();
        }
    }
} 


void
TraceControl::toggleMetrics()
{
    metrics_hidden = !metrics_hidden;
    for (std::vector<QWidget *>::iterator iter = metrics_to_hide.begin(); iter != metrics_to_hide.end(); ++iter)
    {
        QWidget * _w = (*iter);
        MetricTraceWidget * _as_mtw = dynamic_cast<MetricTraceWidget*>(_w);
        if (metrics_hidden)
        {
            _w->hide();
        }
        else
        {
            _w->show();
            if (_as_mtw != NULL)
                _as_mtw->updateScene();
        }
    }
} 




void 
TraceControl::addWidgetToHide(QWidget * _widget)
{
    widgets.push_back( _widget );
}


void 
TraceControl::addMetricsToHide(QWidget * _widget)
{
    metrics_to_hide.push_back(_widget);
    
}

