/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/



#include "blade-config.h"
#include "TraceEventGroupWaitAtBarrierAnalyzer.h"
#include "Service.h"
#include "Trace.h"

extern Trace* trace;

TraceEventGroupWaitAtBarrierAnalyzer::TraceEventGroupWaitAtBarrierAnalyzer()
{

    name = "Wait At Barrier";
    description = "This pattern covers the time\n"
                  "spent waiting in front of\n"
                  "an MPI barrier, which is the\n"
                  "time inside the barrier call\n"
                  "until the last processes has\n"
                  "reached the barrier. ";
    max_min = 0;
    
}

TraceEventGroupWaitAtBarrierAnalyzer::~TraceEventGroupWaitAtBarrierAnalyzer()
{
}


bool
TraceEventGroupWaitAtBarrierAnalyzer::probe(std::vector<TraceEvent*>& events)
{
    bool only_barrier = true;
    for (std::vector<TraceEvent*>::const_iterator iter = events.begin(); iter != events.end(); ++iter)
    {
        TraceEvent * _e = *iter;
        only_barrier &= (_e->get_region_role( ) == OTF2_REGION_ROLE_BARRIER);
    }
    if (!only_barrier)
        return false;
    max_min = 0;
    for (std::vector<TraceEvent*>::iterator iter = events.begin(); iter != events.end(); ++iter)
    {
        max_min = std::max( max_min, (*iter)->get_start_timestamp() );
    } 
    return true;
}

void
TraceEventGroupWaitAtBarrierAnalyzer::analyze(std::vector<TraceEvent*>& events, int dh, int boxwidth )
{
    TraceEventGroupAnalyzer::analyze( events, dh, boxwidth  );
    clear_analyzer();
   


    int i=0;
    for (std::vector<TraceEvent*>::iterator iter = events.begin(); iter != events.end(); ++iter, ++i)
    {
        std::pair<double,double> _dp;
        _dp.first =  ((double)((*iter)->get_start_timestamp() - group_min))/(double)(group_max - group_min);
        _dp.second = ((double)(max_min - group_min))/(double)(group_max - group_min);
	
	TraceEventRect * _rect = new TraceEventRect(max_min -(*iter)->get_start_timestamp(),   trace, (*iter), QRect(_dp.first * boxwidth, location_line_index[i]*dh, ( _dp.second - _dp.first)*boxwidth, dh) );

        QGraphicsSimpleTextItem * _dur_label = new QGraphicsSimpleTextItem( _rect->getDuration() );
        _dur_label->setPos(boxwidth+10,  location_line_index[i]*dh);
        dynamic_labels.push_back(_dur_label); 
        
        
        
	_rect->setPen( QColor(127, 0, 0, 200) );
        QBrush brush;
        brush.setColor( QColor(127, 0, 0, 200)  );
        brush.setStyle( Qt::SolidPattern );
        _rect->setBrush(brush );
                
	rects.push_back( _rect );
    }
}


