/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/



#include "blade-config.h"
#include "TraceEventGroupWaitAtN2NAnalyzer.h"
#include "Service.h"
#include "Trace.h"

extern Trace* trace;

TraceEventGroupWaitAtN2NAnalyzer::TraceEventGroupWaitAtN2NAnalyzer(): TraceEventGroupWaitAtBarrierAnalyzer()
{

    name = "Wait At NxN";
    description = "This pattern covers the time\n"
                  "spent waiting in front of\n"
                  "an collective NxN operatioin,\n"
                  "which is the time inside the \n"
                  "collective call\n"
                  "until the last processes has\n"
                  "reached it. ";
    max_min = 0;
    
}

TraceEventGroupWaitAtN2NAnalyzer::~TraceEventGroupWaitAtN2NAnalyzer()
{
}


bool
TraceEventGroupWaitAtN2NAnalyzer::probe(std::vector<TraceEvent*>& events)
{
 
    bool only_n2n = true;
    for (std::vector<TraceEvent*>::const_iterator iter = events.begin(); iter != events.end(); ++iter)
    {
        TraceEvent * _e = *iter;
        only_n2n &= (_e->get_region_role( ) == OTF2_REGION_ROLE_COLL_ALL2ALL );
    }
    if (!only_n2n)
        return false;
    max_min = 0;
    for (std::vector<TraceEvent*>::iterator iter = events.begin(); iter != events.end(); ++iter)
    {
        max_min = std::max( max_min, (*iter)->get_start_timestamp() );
    } 
    return true;
}

