/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/



#include "blade-config.h"
#include <string>
#include <vector>
#include <inttypes.h>
#include <iostream>
#include <stack>
#include <utility>

#include "TraceEventRect.h"
#include "BladeUtils.h"
#include "Service.h"

TraceEventRect ::TraceEventRect(const Trace * _trace, TraceEvent * _te,  const QRectF & rect, QGraphicsItem * parent): QGraphicsRectItem(rect, parent), te(_te), trace(_trace)
{
    original = true;
    modified_duration = (uint64_t)-1;
}

TraceEventRect ::TraceEventRect(uint64_t _dur, const Trace * _trace, TraceEvent * _te,  const QRectF & rect, QGraphicsItem * parent): QGraphicsRectItem(rect, parent), te(_te), trace(_trace), modified_duration(_dur)
{
    original = false;
}


QString
TraceEventRect ::getRegionName()
{
    std::string name =  te->get_region_name() ;
    return QString::fromStdString( name );
}

QString
TraceEventRect ::getCallPath()
{
    std::vector<std::string> _path;
    std::string result;
    TraceEvent * _p = (TraceEvent*)te;
    while( _p != NULL)
    {
	std::string _s = _p->get_region_name();
	if (_s.compare("") == 0)
		break;
        _path.push_back(BladeUtils::shortCutRegionName(_p->get_region_name())) ;
        _p = _p->get_parent();
    }
    
    for (int i=_path.size()-2; i>=0; --i)
    {
        for (size_t j=0; j<(_path.size()-1 - i);j++)
            result += "  ";
        result += _path[i];
        if (i>0)
            result+="\n";
    }
    
    return QString::fromStdString( result );
}




QString
TraceEventRect ::getStartTime()
{
        std::pair<double, uint16_t> _st = trace->convert_to_seconds( te->get_start_timestamp() );
        return QString("%1%2s").arg(_st.first).arg( QString::fromStdString(bladeservices::convertToUnit (_st.second)) );
}

QString 
TraceEventRect ::getDuration()
{
        
        std::pair<double, uint16_t> _st = trace->convert_to_seconds( (original)?(te->get_end_timestamp() - te->get_start_timestamp()): modified_duration );
        return QString("+%1%2s").arg(_st.first).arg( QString::fromStdString(bladeservices::convertToUnit (_st.second)) );
}
 
    
QString 
TraceEventRect ::getMetrics()
{    
    QString to_return;
    const std::vector<uint64_t>& start_metrics =  te->get_start_metrics( ) ;

    const std::vector<uint64_t>& end_metrics =  te->get_end_metrics( );

    size_t _size = std::min( start_metrics.size(), end_metrics.size());
    
    for (size_t i=0; i< _size; ++i)
    {
        QString _tmp=QString("%1:%2-%3=%4\n").arg(i).arg(end_metrics[i]).arg(start_metrics[i]).arg(end_metrics[i] - start_metrics[i]);
        to_return+= _tmp;
    }
    
    return to_return;
    
}
 
 
 
 
    
TraceEvent *
TraceEventRect ::getTraceEvent()
{
        return te;
}
