/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include "blade-config.h"
#include "TraceLoadEnterFilter.h"



 
TraceLoadEnterFilter::TraceLoadEnterFilter()
{ 
    zoom_enter_filter = new ZoomEnterFilter();
    calllevel_filter = new CallLevelFilter();
}
    

 
TraceLoadEnterFilter::~TraceLoadEnterFilter()
{ 
    delete zoom_enter_filter;
    delete calllevel_filter;
}
    

    
void
TraceLoadEnterFilter::init(uint64_t s_time, uint64_t e_time, int32_t _l)
{
    zoom_enter_filter->unzoom();
    zoom_enter_filter->set_zoom_window( s_time, e_time);
    calllevel_filter->set_call_level( _l );

    zoom_enter_filter->print();
    calllevel_filter->print();
    
}


FilterResult 
TraceLoadEnterFilter::apply( TraceEvent * _te )
{ 
    if (!is_active())
        return FILTER_OK;
    FilterResult result = FILTER_OK;
    
    result = zoom_enter_filter->apply( _te);
    if (result == FILTER_FAIL)
	return FILTER_FAIL;
    
     result = calllevel_filter->apply( _te);
    if (result == FILTER_FAIL)
	return FILTER_FAIL;
    

    return FILTER_OK;
}
    
