/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef __DIVIDE_EVALUATION_CPP
#define __DIVIDE_EVALUATION_CPP 0

#include "blade-config.h"
#include "DivideEvaluation.h"

// using namespace cube;
using namespace std;



DivideEvaluation::DivideEvaluation() : BinaryEvaluation()
{
};

DivideEvaluation::DivideEvaluation( GeneralEvaluation* _arg1,
                                    GeneralEvaluation* _arg2 ) : BinaryEvaluation( _arg1, _arg2 )
{
};


DivideEvaluation::~DivideEvaluation()
{
}


TraceEventStrippedSection
DivideEvaluation::eval(TraceEventSection& _input)
{
    TraceEventStrippedSection denom = argument1->eval(_input);
    double _result1 = 0.;
    double _result2 = 0.;
    TraceEventStrippedSection divid=argument2->eval(_input);
    
    double _d1 = denom.start_value;
    double _d2 = denom.end_value;
    
    double _di1 = divid.start_value;
    double _di2 = divid.end_value;
    
    if ( _d1 != 0. )
    {
        if ( _di1 == 0. )
        {
            if ( _d1 > 0. )
            {
                _result1=DBL_MAX;
            }
            else
            {
                _result1= -DBL_MAX;
            }
        }
        else
        {
            _result1= ( _d1 ) / ( _di1 );
        }
    }
    
        
    if ( _d2 != 0. )
    {
        if ( _di2 == 0. )
        {
            if ( _d2 > 0. )
            {
                _result2=DBL_MAX;
            }
            else
            {
                _result2= -DBL_MAX;
            }
        }
        else
        {
            _result2= ( _d2 ) / ( _di2 );
        }
    }
    divid.end_value = _result2;
    divid.start_value =_result1;
    return divid;
}



#endif
