/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef __DRIVER_CPP
#define __DRIVER_CPP 0

#include "blade-config.h"
#include <vector>
#include <iostream>
#include <sstream>
#include <float.h>
#include <cmath>


#include "Driver.h"
#include "parser.hpp"
#include "scanner.h"
#include "BladeParseContext.h"




using namespace std;

using namespace bladeparser;




Driver::Driver(  )
{
};

Driver::~Driver()
{
};

GeneralEvaluation*
Driver::compile( istream* strin, ostream* errs )
{
//  stringstream strin(cubepl_program);

    BladeParseContext* parseContext = new BladeParseContext(  );
    bladeparser::BladeScanner* lexer  = new bladeparser::BladeScanner( strin, errs, parseContext );
    bladeparser::BladeParser*  parser = new bladeparser::BladeParser( *parseContext, *lexer );
    parser->parse();
    GeneralEvaluation* formula = parseContext->_stack.top();
    parseContext->_stack.pop();

    delete lexer;
    delete parser;
    delete parseContext;

    return formula;
};

bool
Driver::test( std::string& program, std::string& error_message )
{
    stringstream strin( program );
    stringstream ssout;

    BladeParseContext*          parseContext = new BladeParseContext(  true );
    bladeparser::BladeScanner* lexer        = new bladeparser::BladeScanner( &strin, &ssout, parseContext );
    bladeparser::BladeParser*  parser       = new bladeparser::BladeParser( *parseContext, *lexer );
    parser->parse();

    string error_output;
    ssout >> error_output;
    bool _ok = parseContext->syntax_ok;

    if ( error_output.length() != 0 )
    {
        _ok                         = false;
        parseContext->error_message = "BladeScanner cannot recognize token: " + error_output;
    }

    if ( !_ok  )
    {
        error_message = parseContext->error_message;
    }

    delete parseContext->_stack.top();
    parseContext->_stack.pop();
    delete lexer;
    delete parser;
    delete parseContext;

    return _ok;
}

std::string
Driver::printStatus()
{
    return "OK";
};



#endif
