/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#include "blade-config.h"
#include "MetricLine.h"
#include "Service.h"

MetricLine::MetricLine( )
{
    driver = new bladeparser::Driver();
    metric_expression  = QString("delta(metric::PAPI_TOT_CYC())/delta(metric::time())");
     
    std::string derived_metric = metric_expression.toStdString();
    std::stringstream       strin( derived_metric );
    metric = driver->compile( &strin, &std::cerr );
        
}
MetricLine::~MetricLine()
{
    delete driver;
    delete metric;
}

void 
MetricLine::set_expression(QString &_str)
{
    metric_expression = _str;
    std::string derived_metric = metric_expression.toStdString();
    std::stringstream       strin( derived_metric );
    metric = driver->compile( &strin, &std::cerr );
}




std::vector<TraceEventStrippedSection> &
MetricLine::get_stripped_sections(std::vector<TraceEvent*>& _events )
{
    
    std::vector<TraceEventPoint> _points = bladeservices::transform_into_sorted_points(_events);
    
    std::vector<TraceEventSection> _sections = bladeservices::transform_into_sections(_points, z_filter->get_zoom_start(), z_filter->get_zoom_end());
    
    stripped_sections = bladeservices::evaluate_sections(_sections, metric , z_filter->get_zoom_start(), z_filter->get_zoom_end());
    
    return stripped_sections;
}


double
MetricLine::evaluate(uint64_t x)
{
    size_t i = 0;
    for (std::vector<TraceEventStrippedSection>::iterator iter = stripped_sections.begin();
                                                          iter != stripped_sections.end();
                                                          ++iter, ++i)
    {
             TraceEventStrippedSection &_section = (*iter);
             if (_section.end_t> x )
             {
                 double _w = (double)( _section.end_t - _section.start_t);
                 double _d = (double) ( x - _section.start_t);
                 return (_d)/(_w) * ( _section.end_value - _section.start_value) + _section.start_value; 
            }
    }
    
    return 0;
}

