/****************************************************************************
**  Cube callgraph plugin                                                  **
*****************************************************************************
**  Copyright (c) 2014-2020                                                **                                                   **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef CALLGRAPHPLUGIN_H
#define CALLGRAPHPLUGIN_H


#include <QWidget>
#include <QLabel>
#include "PluginServices.h"
#include "CubePlugin.h"
#include "TabInterface.h"
#include "Cube.h"

namespace callgraph_plugin
{
class CallgraphPlugin : public QObject, public cubepluginapi::CubePlugin
{
    Q_OBJECT
    Q_INTERFACES( cubepluginapi::CubePlugin )

#if QT_VERSION >= 0x050000
    Q_PLUGIN_METADATA( IID "Call Graph Plugin" ) // unique PluginName
#endif

public:
    CallgraphPlugin();

    // CubePlugin implementation
    virtual bool
    cubeOpened( cubepluginapi::PluginServices* service );
    void
    cubeClosed();
    virtual QString
    name() const;
    virtual void
    version( int& major,
             int& minor,
             int& bugfix ) const;
    virtual QString
    getHelpText() const;
    virtual QString
    getDeactivationMessage();

    void
    addMenu( QMenu* );

    void
    generateDotFile( const QList<cubepluginapi::TreeItem*>& );


private slots:

    void
    generateDotFileTop();

    void
    generateDotFileSelected();


    void
    contextMenuIsShown( cubepluginapi::TreeType  type,
                        cubepluginapi::TreeItem* item );

private:
    QHash<QString, double>         dotContents;
    cubepluginapi::PluginServices* service;
    cube::CubeProxy*               cube;
    QActionGroup*                  inputGroup, * outputGroup, * omitGroup;
    QStringList                    filterList;

    void
    fillDotHash( cubepluginapi::TreeItem* root );
    double
    computeValue( cubepluginapi::TreeItem* metricItem,
                  cubepluginapi::TreeItem* callItem,
                  cube::CalculationFlavour callState );
};
}

#endif // CALLGRAPHPLUGIN_H
