/****************************************************************************
**  Cube callgraph plugin                                                  **
*****************************************************************************
**  Copyright (c) 2014-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/

#include "callgraph-config.h"
#include "SvgView.h"
#include <QGraphicsSvgItem>
#include <qmath.h>

using namespace callgraph_plugin;

SvgView::SvgView( QWidget* parent ) : QGraphicsView( parent )
{
    setScene( new QGraphicsScene( this ) );
    setTransformationAnchor( AnchorUnderMouse );
    setDragMode( ScrollHandDrag );
    setViewportUpdateMode( FullViewportUpdate );
}

void
SvgView::openFile( QFile& file )
{
    if ( !file.exists() )
    {
        return;
    }

    QGraphicsScene* s = scene();

    s->clear();
    resetTransform();

    svgItem = new QGraphicsSvgItem( file.fileName() );
    svgItem->setFlags( QGraphicsItem::ItemClipsToShape );
    svgItem->setCacheMode( QGraphicsItem::NoCache );
    svgItem->setZValue( 0 );

    s->addItem( svgItem );

    file.close();
}

void
SvgView::wheelEvent( QWheelEvent* event )
{
    qreal factor = qPow( 1.2, event->delta() / 240.0 );
    scale( factor, factor );
    event->accept();
}
