/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2016                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef STACKED_SYSTEM_BARS_PLUGIN_H
#define STACKED_SYSTEM_BARS_PLUGIN_H
#include <QtGui>
#include <QLabel>
#include <QGraphicsScene>
#include <QGraphicsView>
#include <QString>
#include <QWidget>
#include <QList>

#include "PluginServices.h"
#include "CubePlugin.h"
#include "TabInterface.h"
#include "JengaFettToolBar.h"
#include "JengaFettPluginWidget.h"

namespace jengafett_plugin
{
    
    namespace gui 
    {
        class JengaFettPlugin : public QObject, public cubepluginapi::CubePlugin, cubepluginapi::TabInterface
        {
            Q_OBJECT
            Q_INTERFACES( cubepluginapi::CubePlugin )

        #if QT_VERSION >= 0x050000
            Q_PLUGIN_METADATA( IID "JengaFettPlugin" )
        #endif

        public:
            // CubePlugin implementation
            virtual bool
            cubeOpened( cubepluginapi::PluginServices* service );
            virtual void
            cubeClosed();
            virtual QString
            name() const;
            virtual void
            version( int& major,
                    int& minor,
                    int& bugfix ) const;
            virtual QString
            getHelpText() const;

            // TabInterface implementation
            virtual QString
            label() const;
            virtual QIcon
            icon() const;
            virtual QWidget*
            widget();
            virtual void
            valuesChanged();
            virtual QSize
            sizeHint() const;
 
            virtual void
            setActive( bool active );
            

        private slots:
            
            void
            contextMenuIsShown( cubepluginapi::TreeType  type,
                        cubepluginapi::TreeItem* item );
            
            void
            treeItemIsSelected( cubepluginapi::TreeType  type,
                            cubepluginapi::TreeItem* item );
    
            void
            inclMetricToGraph();
            
            void
            clearJengaPlot( );
            
        private:

            
            

            JengaFettPluginWidget*                     widget_;
            JengaFettToolBar*                          toolBar;
            cubepluginapi::TreeItem*       contextItem;
            cubepluginapi::TreeType        contextTreeType;

            cubepluginapi::PluginServices* service;
        };
    };
};
#endif // STACKED_SYSTEM_BARS_PLUGIN_H
