/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/

#include "jengafett-config.h"
#include <iostream>
#include <fstream>

#include <QVBoxLayout>
#include <QPen>
#include <QColor>

#include "JengaFettPluginWidget.h"


using namespace jengafett_plugin::gui;
extern  cubepluginapi::PluginServices* jengafett_service;
 
JengaFettPluginWidget::JengaFettPluginWidget(cube::CubeProxy* _cube, JengaFettToolBar * _toolbar,  QWidget * parent):QWidget(parent),cube(_cube), toolBar(_toolbar)
{
    num_bar_groups = 1;
    
    data = new jengafett_plugin::datalib::CubeDataExtract( cube );
    toolBar->setJengaFettPluginWidget(this);
    
    QVBoxLayout * layout = new QVBoxLayout();
    setLayout( layout );
    customPlot = new QCustomPlot();
    layout->addWidget(customPlot);
    
    
    prime_group = new QCPBarsGroup(customPlot);
    sec_group = new QCPBarsGroup(customPlot);
    logPlot = false;
    
}
 

JengaFettPluginWidget::~JengaFettPluginWidget()
{
    delete data;
}
    
    
double
JengaFettPluginWidget::addBarsToPlot( const  cube::Metric * _met, const  cube::list_of_cnodes&  _cnode, const int Ncall, const int maxNcall)
{
    QCPBars *     bars = new QCPBars(customPlot->xAxis, customPlot->yAxis);
    bars->setWidthType(   QCPBars::wtPlotCoords );
    bars->setWidth(0.2);
    bars->setName(QString::fromStdString( _met->get_disp_name()));
    maxes_y.clear();
    double  local_maxy, global_maxy;
    std::vector<double> _data;   
    std::vector<double> _own_max;   
    data->getData( _met, _cnode, _data, _own_max, global_maxy);
    maxes_y[_met] = global_maxy;
    QVector<double> _data_to_plot;
    std::vector<double>::iterator max_iter = _own_max.begin();
    for (std::vector<double>::iterator iter = _data.begin(); iter != _data.end(); ++iter, ++max_iter)
    {
        switch (displayState)
        {
            case JENGAFETT_COMMON_MAX:
                _data_to_plot.push_back( (*iter)/global_maxy);
                break;
            case JENGAFETT_EVERYBAR_MAX:
                _data_to_plot.push_back( (*iter)/(*max_iter));
                break;
            case JENGAFETT_ABSOLUTE:
            default:
                _data_to_plot.push_back((*iter));
                break;
        }
    }
    
    
    QPen pen;
    pen.setStyle(Qt::SolidLine);
    pen.setColor(QColor::fromHsl((( (254*Ncall/maxNcall)) % 255), 254 % 255, 176 % 255, 255));
    pen.setWidthF(2);
    QBrush  brush;
    brush.setStyle(Qt::SolidPattern);
    brush.setColor(QColor::fromHsl((( (254*Ncall/maxNcall)) % 255), 254 % 255, 176 % 255, 255));
    bars->setPen(pen);
    bars->setBrush(brush); 
            
            
    bars->setData( _x_to_plot, _data_to_plot);
    prime_group->append(bars);
    
    QCPBars *     prev_bars = NULL;
    
    QCPBars *     current_bars = NULL;
    if (_met->num_children() > 0 )
    {
        for ( size_t _im = 0; _im < _met->num_children(); ++_im)
        {
            current_bars = new QCPBars(customPlot->xAxis, customPlot->yAxis);
            current_bars->setWidthType(   QCPBars::wtPlotCoords );
            current_bars->setWidth(0.2);
            _data.clear();
            cube::Metric * _child_met = _met->get_child( _im );
            current_bars->setName(QString::fromStdString( _child_met->get_disp_name()));
            
            
            
            
            data->getData( _child_met, _cnode, _data, _own_max, local_maxy);
            maxes_y[_met] = local_maxy;
            global_maxy = std::max( global_maxy, local_maxy );
            QVector<double> _data_to_plot;
            
            std::vector<double>::iterator max_iter = _own_max.begin();
            for (std::vector<double>::iterator iter = _data.begin(); iter != _data.end(); ++iter, ++max_iter)
            {
                switch (displayState)
                {
                    case JENGAFETT_COMMON_MAX:
                        _data_to_plot.push_back( (*iter)/global_maxy);
                        break;
                    case JENGAFETT_EVERYBAR_MAX:
                        _data_to_plot.push_back( (*iter)/(*max_iter));
                        break;
                    case JENGAFETT_ABSOLUTE:
                    default:
                        _data_to_plot.push_back((*iter));
                        break;
                }
            }
            current_bars->setData( _x_to_plot, _data_to_plot);
            
            QPen pen;
            pen.setStyle(Qt::SolidLine);
            pen.setColor(QColor::fromHsl(254 * (_im)/(_met->num_children()+1) % 255, 254 % 255, 176 % 255, 255));
            pen.setWidthF(2);
            QBrush  brush;
            brush.setStyle(Qt::SolidPattern);
            brush.setColor(QColor::fromHsl(254 * (_im)/(_met->num_children()+1) % 255, 254 % 255, 176 % 255, 255));
            current_bars->setPen(pen);
            current_bars->setBrush(brush); 
            if (prev_bars != NULL)
            {
                current_bars->moveAbove( prev_bars);  
            }
            sec_group->append( current_bars );
            prev_bars = current_bars;
        }
    }
    else if (num_bar_groups==2)
    {
        current_bars = new QCPBars(customPlot->xAxis, customPlot->yAxis);
        current_bars->setWidthType(   QCPBars::wtPlotCoords );
        current_bars->setWidth(0.2);
        current_bars->setName(QString::fromStdString( _met->get_disp_name()));
        _data.clear();
        
        data->getData( NULL, _cnode, _data, _own_max, local_maxy);
        QVector<double> _data_to_plot;
        
        for (std::vector<double>::iterator iter = _data.begin(); iter != _data.end(); ++iter)
        {
            _data_to_plot.push_back(  0.);
        }
        current_bars->setData( _x_to_plot, _data_to_plot);
        
        QPen pen;
        pen.setStyle(Qt::SolidLine);
        pen.setColor(QColor::fromHsl(((254 * Ncall/maxNcall) % 255), 254 % 255, 176 % 255, 255));
        pen.setWidthF(2);
        QBrush  brush;
        brush.setStyle(Qt::SolidPattern);
        brush.setColor(QColor::fromHsl(((254 * Ncall/maxNcall) % 255), 254 % 255, 176 % 255, 255));
        current_bars->setPen(pen); 
        current_bars->setBrush(brush); 
        if (prev_bars != NULL)
        {
            current_bars->moveAbove( prev_bars);  
        }
        sec_group->append( current_bars );
        prev_bars = current_bars;
    }
    return global_maxy;
    
}
    
      
uint16_t
JengaFettPluginWidget::calculateGroupsNumber( const QList<cubepluginapi::TreeItem*>& _metrics ) const
{
     
    uint16_t _num_bar_groups = 1;
    foreach( cubepluginapi::TreeItem* _tm, _metrics)
    { 
        cube::Metric * _m = static_cast<cube::Metric*>(_tm->getCubeObject());
        if ( _m->num_children() > 0)
            _num_bar_groups = 2;
    }
  QList<cube::Metric*> root_metrics =   metric_groups.uniqueKeys ();
    foreach( cube::Metric* _rm, root_metrics )
    {
        QList<cube::Metric*> _values = metric_groups.values( _rm );
        
        foreach( cube::Metric* _metric, _values )
        {
            if ( _metric->num_children() > 0)
            _num_bar_groups = 2;            
        }
    }

    return _num_bar_groups;
    
    
    
}



void
JengaFettPluginWidget::selectedItems(const QList<cubepluginapi::TreeItem*>& _metrics,
                                     const QList<cubepluginapi::TreeItem*>& _cnodes,
                                     const QList<cubepluginapi::TreeItem*>& _sysreses)
{
    num_bar_groups = calculateGroupsNumber( _metrics );
    
    
    customPlot->clearPlottables();
 
    cube::list_of_cnodes cnodes_to_calculate;
    foreach (cubepluginapi::TreeItem* _tc, _cnodes)
    {
        cube::Cnode * _c = static_cast<cube::Cnode*>(_tc->getCubeObject());
        cube::CalculationFlavour _cf = ( _tc->isExpanded())?cube::CUBE_CALCULATE_EXCLUSIVE:cube::CUBE_CALCULATE_INCLUSIVE ;
        
        cnodes_to_calculate.push_back(
            std::make_pair( _c, _cf)
        );
    }
    
    cube::list_of_sysresources sysress_to_calculate;
    foreach (cubepluginapi::TreeItem* _ts, _sysreses)
    {
        cube::Sysres * _s = static_cast<cube::Sysres*>(_ts->getCubeObject());
        cube::CalculationFlavour _sf = ( _ts->isExpanded())?cube::CUBE_CALCULATE_EXCLUSIVE:cube::CUBE_CALCULATE_INCLUSIVE ;
        
        sysress_to_calculate.push_back(
            std::make_pair( _s, _sf)
        );
    }
       
    

    double global_maxy = 0;
    
    std::vector<double> _x;
    QMap< double, QString > _x_labeled;
    data->getArguments(sysress_to_calculate, _x, _x_labeled , maxx);
    _x_to_plot.clear();
    for (std::vector<double>::iterator iter = _x.begin(); iter != _x.end(); ++iter)
    {
        _x_to_plot.push_back( (*iter));
    }   
    
    
    int i= 0;
    foreach( cubepluginapi::TreeItem* _tm, _metrics)
    {
        cube::Metric * _m = static_cast<cube::Metric*>(_tm->getCubeObject());
        
        global_maxy = addBarsToPlot( _m, cnodes_to_calculate, ++i, _metrics.size());
    }
    
// ---------------------- now add bars of additional groups ---------------------------------




    QList<cube::Metric*> root_metrics =   metric_groups.uniqueKeys ();
    foreach( cube::Metric* _rm, root_metrics )
    {
        QList<cube::Metric*> _values = metric_groups.values( _rm );
        
        foreach( cube::Metric* _metric, _values )
        {
            global_maxy = std::max( global_maxy, addBarsToPlot( _metric, cnodes_to_calculate, ++i , _values.size() )) ;
        }
    }

    
    customPlot->xAxis->setRange(-1, maxx+1);
    customPlot->yAxis->setRange(0, (displayState != JENGAFETT_ABSOLUTE)? 1.01:global_maxy);
    
    QSharedPointer<QCPAxisTickerText> textTicker(new QCPAxisTickerText);
    customPlot->xAxis->setTicker(textTicker);
    textTicker->setTicks( _x_labeled ); 
    customPlot->xAxis->setTickLabelRotation(-60);
    customPlot->legend->setVisible(true);
    customPlot->axisRect()->insetLayout()->setInsetAlignment(0, Qt::AlignHCenter|Qt::AlignLeft);
    if (logPlot)
    {
        customPlot->yAxis->setScaleType(QCPAxis::stLogarithmic);
        customPlot->yAxis2->setScaleType(QCPAxis::stLogarithmic);
    }
    else
    {
        customPlot->yAxis->setScaleType(QCPAxis::stLinear);
        customPlot->yAxis2->setScaleType(QCPAxis::stLinear);        
    }
    customPlot->replot();

}


void
JengaFettPluginWidget::addMetricToGraph( cube::Vertex * _v)
{
    cube::Metric * _m = static_cast<cube::Metric*>( _v );
    if ( _m == NULL)
        return;
    
    cube::Metric * _root = _m;
        while ( _root->get_parent() != NULL )
            _root = _root->get_parent();
    
    metric_groups.insert( _root, _m);
}

void
JengaFettPluginWidget::clearJengaPlot( )
{
    metric_groups.clear();
    metric_state.clear();
}
        
        
void 
JengaFettPluginWidget::setAbsolute(  )
{
    displayState = JENGAFETT_ABSOLUTE;
}   


void 
JengaFettPluginWidget::setCommonMax(  )
{
    displayState = JENGAFETT_COMMON_MAX;
}   

void 
JengaFettPluginWidget::setEveryBarMax(  )
{
    displayState = JENGAFETT_EVERYBAR_MAX;
}   



void 
JengaFettPluginWidget::setLogPlot( bool _logplot )
{
    logPlot = _logplot;
    
}
