/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2020                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/

#include "jengafett-config.h"
#include "JengaFettToolBar.h"
#include "PluginServices.h"
#include <QToolButton>
#include <QMessageBox>
#include <QInputDialog>
#include <QMenu>
#include <assert.h>
#include "JengaFettPluginWidget.h"


using namespace jengafett_plugin::gui;
extern  cubepluginapi::PluginServices* jengafett_service;

JengaFettToolBar::JengaFettToolBar( cubepluginapi::PluginServices* service ) : QToolBar( "JengaFett Toolbar" )
{
    parent          = 0;
    this->service   = service;

    logplot_action = this->addAction(QIcon(":/jengafetticons/logPlot.png"),"Show Log Plot");
    connect(logplot_action, SIGNAL(triggered()),
	    this, SLOT(setLogPlot()));   
    
    this->addSeparator();
    
    real_values_action = this->addAction(QIcon(":/jengafetticons/absoluteValues.png"),"Absolute Values");
    connect(real_values_action, SIGNAL(triggered()),
	    this, SLOT(setAbsolute()));   
    
    common_max_action = this->addAction(QIcon(":/jengafetticons/globalMax.png"),"Common Maximum");
    connect(common_max_action, SIGNAL(triggered()),
	    this, SLOT(setCommonMax()));   

    every_bar_max_action = this->addAction(QIcon(":/jengafetticons/everyBarMax.png"),"Own Maximum");
    connect(every_bar_max_action, SIGNAL(triggered()),
	    this, SLOT(setEveryBarMax()));   


    
}


void
JengaFettToolBar::setJengaFettPluginWidget( JengaFettPluginWidget* newparent )
{
    parent = newparent;
}


// slots connected to the actions in the topology tool bar
void
JengaFettToolBar::showSelf()
{
    if ( parent != NULL )
    {
        this->show();
    }
}

void
JengaFettToolBar::hideSelf()
{
    this->hide();
}


void
JengaFettToolBar::refreshPlot()
{
    
    /** Returns selected items in the given tree  @param type METRICTREE, CALLTREE, CALLFLAT, SYSTEMTREE */
    const QList<cubepluginapi::TreeItem*>& metrics = service->getSelections( cubepluginapi::METRICTREE );
    const QList<cubepluginapi::TreeItem*>& cnodes = service->getSelections( cubepluginapi::CALLTREE );
    const QList<cubepluginapi::TreeItem*>& sysres = service->getSelections( cubepluginapi::SYSTEMTREE );
    
    parent->selectedItems( metrics, cnodes, sysres );
}

void
JengaFettToolBar::setAbsolute()
{
    parent->setAbsolute(  );
    refreshPlot();
}

void
JengaFettToolBar::setCommonMax()
{
    parent->setCommonMax(  );
    refreshPlot();
}

void
JengaFettToolBar::setEveryBarMax()
{
    parent->setEveryBarMax(  );
    refreshPlot();
}

void
JengaFettToolBar::setLogPlot()
{
    parent->setLogPlot( true );
    logplot_action->setIcon(QIcon(":/jengafetticons/linPlot.png"));
    logplot_action->setText(tr("Show Linear Plot"));
    disconnect( logplot_action, SIGNAL(triggered()),
	    this, SLOT(setLogPlot()) );
    connect(logplot_action, SIGNAL(triggered()),
	    this, SLOT(setLinearPlot()));  
    refreshPlot();
    
}

void
JengaFettToolBar::setLinearPlot()
{
    parent->setLogPlot( false );
    logplot_action ->setIcon(QIcon(":/jengafetticons/logPlot.png"));
    logplot_action ->setText(tr("Show Log Plot"));
    disconnect( logplot_action, SIGNAL(triggered()),
	    this, SLOT(setLinearPlot()) );
    connect(logplot_action, SIGNAL(triggered()),
	    this, SLOT(setLogPlot()));  
    refreshPlot();
}





