/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2016                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef SCOREPION_PLUGIN_H
#define SCOREPION_PLUGIN_H
#include <QtGui>
#include <QLabel>
#include "PluginServices.h"
#include "CubePlugin.h"
#include "TabInterface.h"
#include "ScorePionPluginWidget.h"

namespace scorepion_plugin
{
    
    namespace misc
    {
        class ScorePionConfigFile;
    };
    namespace filters
    {
        class ScorePionFiltersFactory;
    };
    
    namespace gui 
    {
                    
class ScorePionToolBar;

class ScorePionPlugin : public QObject, public  cubepluginapi::CubePlugin,  cubepluginapi::TabInterface,  cubepluginapi::SettingsHandler
{
    Q_OBJECT
    Q_INTERFACES(  cubepluginapi::CubePlugin )

#if QT_VERSION >= 0x050000
    Q_PLUGIN_METADATA( IID "ScorePionPlugin" )
#endif

public:
    // CubePlugin implementation
    virtual bool
    cubeOpened(  cubepluginapi::PluginServices* service );
    virtual void
    cubeClosed();
    virtual QString
    name() const;
    virtual void
    version( int& major,
             int& minor,
             int& bugfix ) const;
    virtual QString
    getHelpText() const;

    // TabInterface implementation
    virtual QString
    label() const;
    virtual QIcon
    icon() const;
    virtual QWidget*
    widget();
    virtual QSize
    sizeHint() const;
    
    virtual void
    valuesChanged();
    
    
    // SettingsHandler implementation
    virtual void
    loadGlobalSettings( QSettings& settings );
    virtual void
    saveGlobalSettings( QSettings& settings );
    virtual QString
    settingName();
    
    virtual void
    setActive( bool active );

private slots:
    void
    contextMenuIsShown( cubepluginapi::TreeType  type,
                        cubepluginapi::TreeItem* item );
    void
    inclRegNameToFilter();
    
    void
    inclRegFileToFilter();

    void
    exclRegNameToFilter();
    
    void
    exclRegFileToFilter();

    
private:

    
    
    ScorePionPluginWidget *                     widget_;
    ScorePionToolBar*                          toolBar;

    cubepluginapi::TreeItem*       contextItem;
    cubepluginapi::TreeType        contextTreeType;
    
    cubepluginapi::PluginServices* service;
     
    scorepion_plugin::filters::ScorePionFiltersFactory         * filter_factory;
    scorepion_plugin::misc::ScorePionConfigFile             * config_file;

    bool active;
};

    };
};

#endif // SCOREPION_PLUGIN_H
