/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2019                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef SCOREPION_PLUGIN_WIDGET_H
#define SCOREPION_PLUGIN_WIDGET_H
#include <list>
#include <QtGui>
#include <QLabel>
#include <QWidget>
#include <QVBoxLayout>
#include <QToolBar>
#include <QStackedWidget>
#include <QTabWidget>
#include <QTextEdit>
#include <QPushButton>

#include "CubeProxy.h"
#include "PluginServices.h"
#include "TabInterface.h"
#include "FiltersChain.h"
#include "CubePL2MemoryManager.h"

namespace scorepion_plugin
{
    
    namespace filters
    {
        class ScorePionFiltersFactory;
    }
    namespace gui 
    {
        
                
        class    ScorePionToolBar;
        class ScorePionFilterSetWidget;
    
        class ScorePionPluginWidget : public QWidget
        {
            Q_OBJECT
            
        private:
            
            
            ScorePionToolBar*                          toolBar;
            
            cube::CubeProxy*    cube;
            cubepluginapi::TreeItem* reduced_trace_size_metric;
            cubepluginapi::TreeItem* total_trace_size_metric;
            cubepluginapi::TreeItem* reduced_overhead_metric;
            cubepluginapi::TreeItem* total_overhead_metric;
            
            
            scorepion_plugin::filters::ScorePionFiltersFactory * filter_factory;
            scorepion_plugin::filters::FiltersChain  * filters ;
            
            
            std::list<ScorePionFilterSetWidget*> filter_sets;
            QVBoxLayout* main_layout;
            QLabel * full_trace_size;
            QLabel * reduced_trace_size;
            QLabel * scorep_total_memory;
            QLabel * scorep_overhead;
                
            double total_runtime;
            
            
            QList<const cubepluginapi::TreeItemMarker*> markerList; 
            
            cube::CubePLMemoryManager * cubepl_memory;
            cube::CubePL2MemoryAdress variable_adress;
            cube::CubePL2MemoryAdress region_bytes_adress;
            cube::CubePL2MemoryAdress region_overhead_adress;
            
            
            void
            defineTreeItemMarker();
    
            void
            updateMarkers();
            
            
            double
            getRegionId( const cube::Vertex *) const;
            
        private slots:
            
            void
            applyFilters();
            

            
        public slots:
                   
            void
            writeFilterFile( ); 
            
            void
            loadFilterFile( );        
            
            void
            addedMetric();
            
            void
            adjustCountersNum( int );          
            
            
        public:
            ScorePionPluginWidget(cube::CubeProxy*, ScorePionToolBar*,  scorepion_plugin::filters::ScorePionFiltersFactory * filter_factory,  QWidget * parent = NULL);
            ~ScorePionPluginWidget();
            
            
            void
            removeFilterSetWidget( ScorePionFilterSetWidget * ); 
            
            void
            addDefaultFilterRule( );
            
            // ----- used by contex menu 
            void
            addIncludeFile( cube::Vertex *  );

            void
            addIncludeRegion( cube::Vertex * );

            void
            addExcludeFile( cube::Vertex * );

            void
            addExcludeRegion( cube::Vertex * );
            
            // used by toolbar buttons 
            void
            addIncludeFile( std::vector<cube::Vertex *> &  );

            void
            addIncludeRegion( std::vector<cube::Vertex *> & );

            void
            addExcludeFile( std::vector<cube::Vertex *> & );

            void
            addExcludeRegion( std::vector<cube::Vertex *> & );
             
            // actual call
            void
            addFilterRule( scorepion_plugin::filters::FilterRulesSet * filter_rules_set);
                         
            void
            updateMeasurementLabels();
        };
    };
};
#endif // TESTPLUGIN1_H
